/* -*- mode: C; c-file-style: "Stroustrup" -*-
 *
 * das08jr
 * =======
 *
 * Linux device driver for Computer Boards CIO-DAS08/Jr-AO board.
 *
 * This board has 8 analog input lines and 2 analog output lines. Additionally
 * there are 8 digital outputs.
 * These lines are mapped onto minor 0 (digital output), 1 - 8 for analog
 * input, and 1 + 2 for analog output.
 *
 * Copyright (C) 1998 Jochen Kpper
 *
 * $Id: das08jr.c,v 1.1 1998/11/08 16:18:15 jochen Exp $
 */


#include <linux/config.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/version.h>

#include <asm/io.h>
#include <asm/system.h>


/* delay for A/D- and D/A-conversions [us] */
#define DAS08JR_DELAY ( 30 )


/* Module Name */
#define DAS08JR_NAME "das08jr"




/* driver version */
#if 0
static const char das08jr_version[] = "PCI-DAS08Jr-AO driver ($Id: das08jr.c,v 1.1 1998/11/08 16:18:15 jochen Exp $)";
#endif





/* Port Offsets */
#define DAS08JR_SIZE                            ( 8 ) /* number of ports */
#define DAS08JR_ANALOG_IN_LSB                   ( 0 ) /* read */
#define DAS08JR_ANALOG_IN_MSB                   ( 1 ) /* read */
#define DAS08JR_START                           ( 1 )
#define DAS08JR_STATUS                          ( 2 ) /* read */
#define DAS08JR_MUX                             ( 2 )
#define DAS08JR_CONTROL                         ( 2 )
#define DAS08JR_DIGITAL_IN                      ( 3 ) /* read */
#define DAS08JR_DIGITAL_OUT                     ( 3 )
#define DAS08JR_ANALOG_OUT_LSB0                 ( 4 )
#define DAS08JR_ANALOG_OUT_MSB0                 ( 5 )
#define DAS08JR_ANALOG_OUT_LSB1                 ( 6 )
#define DAS08JR_ANALOG_OUT_MSB1                 ( 7 )



/* number of available lines */
#define DAS08JR_ANALOG_INPUTS   ( 8 )
#define DAS08JR_ANALOG_OUTPUTS  ( 2 )
#define DAS08JR_DIGITAL_LINES   ( 1 )
#define DAS08JR_MAX_LINE        ( 8 ) /* we have nine lines starting at zero */





#if 0

/**
 * Open device.
 *
 * @author Jochen Kpper
 * @version $Id: das08jr.c,v 1.1 1998/11/08 16:18:15 jochen Exp $
 */ 
static int das08jr_open( struct inode *inode, struct file *fp )
{
    int minor = MINOR( inode->i_rdev );
    if( DAS08JR_MAX_LINE < minor )
	return( -ENODEV );
    if( DAS08JR_DIGITAL_LINES > minor ) {
	/* digital lines */
	fp->f_op = &das08jr_fops_digital;
    } else if( DAS08JR_ANALOG_OUTPUTS + DAS08JR_DIGITAL_LINES > minor ) {
	/* readonly analog lines */
	if( FMODE_WRITE & fp->f_mode )
	    return( -EACCES );
	fp->f_op = &das08jr_fops_ro;
    }
    MOD_INC_USE_COUNT;
    return( 0 );
}



/**
 * Close device
 *
 * @author Jochen Kpper
 * @version $Id: das08jr.c,v 1.1 1998/11/08 16:18:15 jochen Exp $
 */ 
static void das08jr_release( struct inode *inode, struct file *fp )
{
    MOD_DEC_USE_COUNT;
    return;
}

#endif


/**
 * Read digital lines
 *
 * @author Jochen Kpper
 * @version $Id: das08jr.c,v 1.1 1998/11/08 16:18:15 jochen Exp $
 */
static int das08jr_read_digital( comedi_device *dev, comedi_trig *it )
{
	it->data= inb( dev->iobase + DAS08JR_DIGITAL_IN );
	
	return 0;
}



/**
 * write digital lines
 *
 * @author Jochen Kpper
 * @version $Id: das08jr.c,v 1.1 1998/11/08 16:18:15 jochen Exp $
 */
static int das08jr_write_digital( struct inode *inode, struct file *fp, const char *buf, int count )
{
    outb( it->data , dev->iobase + DAS08JR_DIGITAL_OUT );
    return( 0 );
}



/**
 * Read from analog line
 *
 * @author Jochen Kpper
 * @version $Id: das08jr.c,v 1.1 1998/11/08 16:18:15 jochen Exp $
 */
static int das08jr_read( struct inode *inode, struct file *fp, char *buf, int count )
static int das08jr_read( comedi_device *dev, comedi_trig *it )
{
	unsigned char lsb,msb;s

    /* we alway read one short value */

	/* start daq */
	
	/* DS: shouldn't we do something here to trigger it? */
	
	/* busy waiting for board to finish */
	/* udelay( DAS08JR_DELAY ); */
	/* read data and put it into user space */
	lsb = inb( DAS08JR_ANALOG_IN_LSB );
	msb = inb( DAS08JR_ANALOG_IN_MSB );
	
	it->data = ( ( (int)msb ) << 4 ) + ( lsb >> 4 );
	
    	return( 0 );
}


/*------------------------------*/

/**
 * Write to writeable analog line.
 *
 * We do not need to check wether this really is an writeable line, since open
 * cares about that an we only get called on lines 1 or 2.
 *
 * We simply put one value to the output ports, start D/A conversion, and return.
 *
 * @return On success the number of written bytes ( always 2 ),
 *         a negative error code otherwise.
 *
 * @author Jochen Kpper
 * @version $Id: das08jr.c,v 1.1 1998/11/08 16:18:15 jochen Exp $
 */
static int das08jr_write_rw( struct inode *inode, struct file *fp, const char *buf, int count )
{
    /* we need a short value */
    if( count >= 2 ) {
	unsigned short val = get_user( (unsigned short *)buf );
	if( DAS08JR_ANALOG_OUTPUTS == MINOR( inode->i_rdev ) ) {
	    outb( (unsigned char)val >> 4, DAS08JR_ANALOG_OUT_MSB1 );
	    outb( (unsigned char)val << 4, DAS08JR_ANALOG_OUT_LSB1 );
	} else {
	    outb( (unsigned char)val >> 4, DAS08JR_ANALOG_OUT_MSB0 );
	    outb( (unsigned char)val << 4, DAS08JR_ANALOG_OUT_LSB0 );
	}
	outb( 0xff, DAS08JR_START );
    }
    return( 0 );
}



/**
 * Write to readonly analog line.
 *
 * @return Since we are always called on readonly lines, simply return -EPERM.
 *
 * @author Jochen Kpper
 * @version $Id: das08jr.c,v 1.1 1998/11/08 16:18:15 jochen Exp $
 */
static int das08jr_write_ro( struct inode *inode, struct file *fp, const char *buf, int count )
{
    return( -EPERM );
}



/* implemented file operations for digital lines */
static struct file_operations das08jr_fops_digital =
{
    NULL,
    das08jr_read_digital,
    das08jr_write_digital,
    NULL,
    NULL,
    NULL,
    NULL,
    das08jr_open,
    das08jr_release,
    /* no more */
};
    
/* implemented file operations for readonly analog lines ( 3 - 8 ) */
static struct file_operations das08jr_fops_ro =
{
    NULL,
    das08jr_read,
    das08jr_write_ro,
    NULL,
    NULL,
    NULL,
    NULL,
    das08jr_open,
    das08jr_release,
    /* no more */
};
    
/* implemented file operations for writable analog lines( 1 + 2 ) */
static struct file_operations das08jr_fops_rw =
{
    NULL,
    das08jr_read,
    das08jr_write_rw,
    NULL,
    NULL,
    NULL,
    NULL,
    das08jr_open,
    das08jr_release,
    /* no more */
};


    
/**
 * Register ports,
 * register as character device,
 * export exactly the symbols we want to be public.
 *
 * @return Success status, zero on success.
 *
 * @author Jochen Kpper
 * @version $Id: das08jr.c,v 1.1 1998/11/08 16:18:15 jochen Exp $
 */
int init_module( void )
{
    int result = 0;
    if( 0 == io ) {
	/* should autoprobe here */
	io = DAS08JR_PORT;
    }
    if( -EBUSY == ( result = check_region( io, DAS08JR_SIZE ) ) ) {
	printk( KERN_WARNING DAS08JR_NAME ": Cannot register port memory at %lx.\n", io );
    } else {
	request_region( io, DAS08JR_SIZE, DAS08JR_NAME );
	if( 0 > ( result = register_chrdev( DAS08JR_MAJOR, DAS08JR_NAME, &das08jr_fops_rw ) ) ) {
	    printk( KERN_WARNING DAS08JR_NAME ": Cannot register major number %d.\n", DAS08JR_MAJOR );
	    release_region( DAS08JR_PORT, DAS08JR_SIZE );
	} else {
	    register_symtab( NULL );
	    printk( KERN_INFO DAS08JR_NAME " (%s) loaded.\nCopyright (C) 1998 Jochen K\"upper.\n", das08jr_version );
	}
    }
    return( result );
}



/**
 * @author Jochen Kpper
 * @version $Id: das08jr.c,v 1.1 1998/11/08 16:18:15 jochen Exp $
 */ 
void cleanup_module( void )
{
    unregister_chrdev( DAS08JR_MAJOR, DAS08JR_NAME );
    release_region( DAS08JR_PORT, DAS08JR_SIZE );
    printk( KERN_INFO DAS08JR_NAME " unloaded.\n" );
    return;
}
